@@ -35,12 +35,11 @@
 #' tgrid <- seq(0, 10, length = 1001)
 #' ## make it irregularly spaced
 #' tgrid <- sort(sample(tgrid, 800))
-#' dat <- rbmme(tgrid, 1, 1)
+#' dat <- rBMME(tgrid, 1, 1)
 #' plot(dat[,1], dat[,2], xlab="t", ylab="X(t)", type="l")
 #'
 #' @export
-
-rbmme <- function(time, dim = 2,  sigma = 1, delta = 1) {  
+rBMME <- function(time, dim = 2,  sigma = 1, delta = 1) {  
     n <- length(time)
     dat <- matrix(NA_real_, n, dim)
     t.inc.root <- sqrt(diff(time))
@@ -51,7 +50,15 @@ rbmme <- function(time, dim = 2,  sigma = 1, delta = 1) {
         err <- rnorm(n, 0, sd = delta[i])
         dat[,i] <- bm + err
     }
-    cbind(time = time, dat)
+    as.data.frame(cbind(time = time, dat))
+}
+
+#' 'rBmme' is deprecated. Using new function 'rBMME' instead.
+#' @rdname rBMME
+#' @export
+rBmme <- function(time, dim = 2,  sigma = 1, delta = 1) {
+    .Deprecated("rBMME")
+    rBMME(time, dim, sigma, delta)
 }
 
 #### generate the covariance matrix in sparse structure
@@ -128,14 +135,26 @@ bmme.start <- function(dat) {
 #' Fit a Brownian Motion with Measurement Error
 #'
 #' Given discretely observed animal movement locations, fit a Brownian
-#' motion model with measurement errors.
+#' motion model with measurement errors. Using \code{segment} to fit
+#' part of observations to the model. A practical application of this
+#' feature is seasonal analysis.
 #'
 #' @param data a data.frame whose first column is the observation time, and other
-#'     columns are location coordinates.
+#'     columns are location coordinates. If \code{segment} is not \code{NULL},
+#'     additional column with the same name given by \code{segment} should be
+#'     included. This additional column is used to indicate which part of
+#'     observations shoule be used to fit model. The value of this column can
+#'     be any integer with 0 means discarding this observation and non-0 means
+#'     using this obversvation. Using different non-zero numbers indicate different
+#'     segments. (See vignette for more details.)
+#'     
 #' @param start starting value of the model, a vector of two component, one for
 #'     sigma (sd of BM) and the other for delta (sd for measurement error).
 #'     If unspecified (NULL), a moment estimator will be used assuming equal
 #'     sigma and delta.
+#' @param segment character variable, name of the column which indicates segments,
+#'     in the given \code{data.frame}. The default value, \code{NULL}, means using
+#'     whole dataset to fit the model.
 #' @param method the method argument to feed \code{optim}.
 #' @param optim.control a list of control that is passed down to \code{optim}.
 #'
@@ -146,7 +165,7 @@ bmme.start <- function(dat) {
 #'
 #' @return
 #'   A list of the following components:
-#'   \item{estimate }{the esimated parameter vector}
+#'   \item{estimate }{the estimated parameter vector}
 #'   \item{var.est }{variance matrix of the estimator}
 #'   \item{loglik }{loglikelihood evaluated at the estimate}
 #'   \item{convergence}{convergence code from optim}
@@ -156,28 +175,67 @@ bmme.start <- function(dat) {
 #' On modeling animal movements using Brownian motion with measurement
 #' error. Ecology 95(2): p247--253. doi:doi:10.1890/13-0532.1.
 #' @seealso
-#'   \code{\link{fitMovRes}}
+#'   \code{\link{fitMR}}
 #' @examples
 #' set.seed(123)
 #' tgrid <- seq(0, 500, by = 1)
-#' dat <- rbmme(tgrid, sigma = 1, delta = 0.5)
-#' fit <- fitBmme(dat)
+#' dat <- rBMME(tgrid, sigma = 1, delta = 0.5)
+#'
+#' ## using whole dataset to fit BMME
+#' fit <- fitBMME(dat)
 #' fit
+#'
+#' ## using part of dataset to fit BMME
+#' batch <- c(rep(0, 100), rep(1, 200), rep(0, 50), rep(2, 100), rep(0, 51))
+#' dat.segment <- cbind(dat, batch)
+#' fit.segment <- fitBMME(dat.segment, segment = "batch")
+#' head(dat.segment)
+#' fit.segment
+#' 
 #' @export
+fitBMME <- function(data, start = NULL, segment = NULL,
+                    method = "Nelder-Mead",
+                    optim.control = list()) {
+    if (is.null(segment)) {
 
+
+        ## normal process
+        if (is.null(start)) start <- bmme.start(data)
+        dinc <- apply(data, 2, diff)
+        fit <- optim(start, nllk.bmme, dinc = dinc, hessian = TRUE,
+                     method=method, control = optim.control)
+        ## Sigma <- getSparseSigma(data[,1], fit$par)
+        ans <- list(estimate = fit$par,
+                    var.est = solve(fit$hessian),
+                    loglik = - fit$value,
+                    convergence = fit$convergence
+                    ## vmat.l = t(chol(Sigma))
+                    )
+        return(ans)
+
+        
+    } else {
+
+
+        ## seasonal process
+        result <- fitBMME_seasonal(data = data, segment = segment,
+                                   start = start, method = method,
+                                   optim.control = optim.control)
+        return(result)
+
+        
+    }
+}
+
+
+#' 'fitBmme' is deprecated. Using new function 'fitBMME' instead.
+#' @rdname fitBMME
+#' @export
 fitBmme <- function(data, start = NULL, method = "Nelder-Mead",
                     optim.control = list()) {
-    if (is.null(start)) start <- bmme.start(data)
-    dinc <- apply(data, 2, diff)
-    fit <- optim(start, nllk.bmme, dinc = dinc, hessian = TRUE, method=method, control = optim.control)
-    ## Sigma <- getSparseSigma(data[,1], fit$par)
-    ans <- list(estimate = fit$par,
-                var.est = solve(fit$hessian),
-                loglik = - fit$value,
-                convergence = fit$convergence
-                ## vmat.l = t(chol(Sigma))
-                )
-    ans
+    .Deprecated("fitBMME")
+    fitBMME(data = data, start = start, segment = NULL,
+            method = method, optim.control = optim.control)
 }
 
 ## remind Vladmir to check linear transformation of (B0, ... Bn, xi0, ..., xin)
@@ -270,7 +328,7 @@ doccuTime <- function(x, tint, param, dat) {
     if (!is.matrix(x)) x <- as.matrix(x)
     SigmaZ <- getVarObs(dat[,1], param)
     SigmaZ.l <- t(chol(SigmaZ))
-    doccu <- dbmme.x.v(x, param, SigmaZ.l, dat)
+    doccu <- dbmme.x.v(x, tint, param, SigmaZ.l, dat)
     cbind(x, doccu)
 }
 
